@@ -125,7 +125,11 @@ void Model::responseReceived()
             break;
         }
     }
-    it->idleLauncher->postponeIdleIfActive();
+
+    if ( it->idleLauncher ) {
+        // The parser might be already dying, and killParser() tries hard to kill IdleLauncher, too
+        it->idleLauncher->postponeIdleIfActive();
+    }
 }
 
 void Model::handleState( Imap::Parser* ptr, const Imap::Responses::State* const resp )
@@ -1352,6 +1356,9 @@ void Model::broadcastParseError( const uint parser, const QString& exceptionClas
 {
     emit logParserFatalError( parser, exceptionClass, errorMessage, line, position );
     QByteArray details = ( position == -1 ) ? QByteArray() : QByteArray( position, ' ' ) + QByteArray("^ here");
+    qDebug() << "ERROR:" << exceptionClass;
+    qDebug() << "MESSAGE:" << errorMessage;
+    qDebug() << "DETAILS:" << QString(line).left(position) << "[[HERE]]" << QString(line).right(position);
     emit connectionError( trUtf8( "<p>The IMAP server sent us a reply which we could not parse. "
                                   "This might either mean that there's a bug in Trojiá's code, or "
                                   "that the IMAP server you are connected to is broken. Please "
@@ -1389,20 +1396,32 @@ void Model::switchToMailbox( const QModelIndex& mbox, const RWMode mode )
 {
     if ( ! mbox.isValid() )
         return;
-
+    
     if ( _netPolicy == NETWORK_OFFLINE )
         return;
-
+    
     if ( TreeItemMailbox* mailbox = dynamic_cast<TreeItemMailbox*>(
-                 realTreeItem( mbox ) ) ) {
+        realTreeItem( mbox ) ) ) {
         Parser* ptr = _getParser( mailbox, mode );
         if ( _parsers[ ptr ].capabilitiesFresh &&
-             _parsers[ ptr ].capabilities.contains( QLatin1String( "IDLE" ) ) ) {
+            _parsers[ ptr ].capabilities.contains( QLatin1String( "IDLE" ) ) ) {
             _parsers[ ptr ].idleLauncher->enterIdleLater();
         }
     }
 }
 
+void Model::switchToMailbox( TreeItemMailbox* mbox, const RWMode mode )
+{
+    if ( _netPolicy == NETWORK_OFFLINE )
+        return;
+    
+    Parser* ptr = _getParser( mbox, mode );
+    if ( _parsers[ ptr ].capabilitiesFresh &&
+        _parsers[ ptr ].capabilities.contains( QLatin1String( "IDLE" ) ) ) {
+        _parsers[ ptr ].idleLauncher->enterIdleLater();
+    }
+}
+
 void Model::enterIdle( Parser* parser )
 {
     noopTimer->stop();
