@@ -34,7 +34,7 @@
 namespace Imap {
 namespace Mailbox {
 
-MsgListModel::MsgListModel( QObject* parent, Model* model ): QAbstractProxyModel(parent), msgList(0)
+MsgListModel::MsgListModel( QObject* parent, Model* model ): QAbstractProxyModel(parent), msgList(0), waitingForMessages(false)
 {
     setSourceModel( model );
 
@@ -428,24 +428,30 @@ void MsgListModel::handleRowsInserted( const QModelIndex& parent, int start, int
     if ( msgList && msgList == newList ) {
         endInsertRows();
     }
+
+    if ( waitingForMessages ) {
+        waitingForMessages = false;
+        emit messagesAvailable();
+    }
 }
 
 void MsgListModel::setMailbox( const QModelIndex& index )
 {
+    waitingForMessages = true;
     if ( ! index.isValid() ) {
         msgList = 0;
         reset();
         emit mailboxChanged();
         return;
     }
-
+    
     const Model* model = 0;
     TreeItemMailbox* mbox = dynamic_cast<TreeItemMailbox*>( Model::realTreeItem( index, &model ));
     Q_ASSERT( mbox );
     if ( ! mbox->isSelectable() )
         return;
     TreeItemMsgList* newList = dynamic_cast<TreeItemMsgList*>(
-            mbox->child( 0, const_cast<Model*>( model ) ) );
+    mbox->child( 0, const_cast<Model*>( model ) ) );
     Q_ASSERT( newList );
     if ( newList != msgList && mbox->isSelectable() ) {
         msgList = newList;
@@ -460,6 +466,23 @@ void MsgListModel::setMailbox( const QModelIndex& index )
     }
 }
 
+void MsgListModel::setMailbox( TreeItemMailbox* mbox, Model* model )
+{
+    waitingForMessages = true;
+    TreeItemMsgList* msgList = dynamic_cast<TreeItemMsgList*>( mbox->child( 0, const_cast<Model*>( model ) ) );
+    if ( msgList != this->msgList ) {
+        this->msgList = msgList;
+        reset();
+        emit mailboxChanged();
+        // We want to tell the Model that it should consider starting the IDLE command.
+        // We do not strictly need the RW access here, but given that we're probably
+        // going to upgrade to RW anyway (at least when marking the message as "Seen")
+        // and that re-syncing FLAGS is rather expensive, it probably makes much sense
+        // to just open the mailbox RW right now.
+        model->switchToMailbox( mbox, Imap::Mailbox::Model::ReadWrite );
+    }
+}
+
 TreeItemMailbox* MsgListModel::currentMailbox() const
 {
     return msgList ? dynamic_cast<TreeItemMailbox*>( msgList->parent() ) : 0;
